@@ -9,23 +9,25 @@ class SGraph:
         self.p = None
         self.pi = None
 
-    def __copy__(self): #todo don't work
+    def __copy__(self):  # todo don't work
         return SGraph(self.gr.copy())
 
+    def clean_copy(self):
+        G = self.gr.__class__()
+        G.add_edges_from(self.gr.edges())
+        return SGraph(G)
+
     def directed(self):
         return nx.is_directed(self.get_nx_graph())
 
+    def number_of_edges(self):
+        return self.gr.number_of_edges()
+
     def clean(self):
         self.gr.clear()
 
     def subgraph(self, nodes):
-        sub = self.gr.__class__()
-        sub.add_nodes_from(nodes)
-
-        for node1 in nodes:
-            for node2 in nodes:
-                if self.gr.has_edge(node1, node2):
-                    sub.add_edge(node1, node2)
+        sub = self.gr.subgraph(nodes).copy()
         return SGraph(sub)
 
     def get_ids(self, stable=False):
@@ -99,8 +101,9 @@ class SGraph:
 
     def get_stat_dist(self):
         if self.pi is None:
-            A = np.subtract(np.transpose(self.get_prob_matrix()),
-                        np.diag(np.ones(self.size())))
+            A = np.subtract(
+                np.transpose(self.get_prob_matrix()), np.diag(np.ones(self.size()))
+            )
             A = np.vstack([A, np.ones(self.size())])
 
             b = np.zeros(self.size() + 1)
@@ -169,7 +172,11 @@ class SGraph:
     def get_in_degrees(self, ind, un=True, w=None):
         if self.directed():
             try:
-                res = [x[0] for x in self.gr.in_edges(ind) if self.get_edge_data(x[0], x[1], w) > 0]
+                res = [
+                    x[0]
+                    for x in self.gr.in_edges(ind)
+                    if self.get_edge_data(x[0], x[1], w) > 0
+                ]
             except:
                 res = [x[0] for x in self.gr.in_edges(ind)]
 
@@ -178,12 +185,23 @@ class SGraph:
             else:
                 return np.array(res)
         else:
-            return self.get_degrees(ind, un=un, w=w)
+            res = [x[0] for x in self.gr.edges(ind) if x[0] != ind] + [
+                x[1] for x in self.gr.edges(ind) if x[1] != ind
+            ]
+
+            if un:
+                return np.unique(res)
+            else:
+                return np.array(res)
 
     def get_out_degrees(self, out, un=True, w=None):
         if self.directed():
             try:
-                res = [x[1] for x in self.gr.out_edges(out) if self.get_edge_data(x[0], x[1], w) > 0]
+                res = [
+                    x[1]
+                    for x in self.gr.out_edges(out)
+                    if self.get_edge_data(x[0], x[1], w) > 0
+                ]
             except:
                 res = [x[1] for x in self.gr.out_edges(out)]
 
@@ -192,22 +210,14 @@ class SGraph:
             else:
                 return np.array(res)
         else:
-            return self.get_degrees(out, un=un, w=w)
+            res = [x[0] for x in self.gr.edges(out) if x[0] != out] + [
+                x[1] for x in self.gr.edges(out) if x[1] != out
+            ]
 
-    def get_degrees(self, ind, un=True, w=None):
-        try:
-            res = [x[0] for x in self.gr.edges(ind) if
-                   self.get_edge_data(x[0], x[1], w) > 0 and x[0] != ind] + [x[1] for x in self.gr.edges(ind) if
-                                                                             self.get_edge_data(x[0], x[1], w) > 0 and
-                                                                             x[1] != ind]
-        except:
-            res = [x[0] for x in self.gr.edges(ind) if x[0] != ind] + [x[1] for x in self.gr.edges(ind) if
-                                                                       x[1] != ind]
-
-        if un:
-            return np.unique(res)
-        else:
-            return np.array(res)
+            if un:
+                return np.unique(res)
+            else:
+                return np.array(res)
 
     def get_data(self, attrs=None, nodes=None):
         if nodes is None:
@@ -236,7 +246,7 @@ class SGraph:
         data.to_csv(path, sep=sep, index=False)
 
     def del_attrs(self, attrs):
-        if not hasattr(attrs, '__iter__'):
+        if not hasattr(attrs, "__iter__"):
             attrs = [attrs]
 
         for n in self.get_ids():
@@ -250,10 +260,7 @@ class SGraph:
         count = 0
         if typ == "sl" or typ == "sl-ext":
             for j in nodes:
-                if self.directed():
-                    subs = self.get_in_degrees(j)
-                else:
-                    subs = self.get_degrees(j)
+                subs = self.get_in_degrees(j)
                 if typ == "sl-ext":
                     subs = np.append(j, subs)
                 if part:
@@ -351,7 +358,11 @@ class SGraph:
         for node in nodes:
             if root == node:
                 continue
-            path = next(nx.all_shortest_paths(self.gr, source=root, target=node, method="bellman-ford"))
+            path = next(
+                nx.all_shortest_paths(
+                    self.gr, source=root, target=node, method="bellman-ford"
+                )
+            )
             if isinstance(path[0], list):
                 for path_ in path:
                     pathes.append(np.array(path_))
@@ -375,7 +386,7 @@ class SGraph:
 
             step_ = []
             for st in step:
-                for res in self.get_degrees(st[-1]):
+                for res in self.get_in_degrees(st[-1]):
                     if res not in done:
                         done.append(res)
                         st_ = st.copy()
@@ -403,7 +414,7 @@ class SGraph:
 
             step_ = []
             for st in step:
-                for res in self.get_degrees(st[-1]):
+                for res in self.get_in_degrees(st[-1]):
                     if res not in done:
                         done.append(res)
                         st_ = st.copy()
