@@ -28,7 +28,7 @@ class Creator:
         gr = SGraph(nx.DiGraph())
         gr.add_nodes(np.arange(nm))
 
-        for i in (np.arange(nm)):
+        for i in np.arange(nm):
             if ind_d[i] == 0:
                 continue
             i_s = np.setdiff1d(np.arange(nm), i)
@@ -78,7 +78,7 @@ class Creator:
         return res
 
     @staticmethod
-    def simple_graph(network, pre="../"):
+    def simple_graph(network, pre="../", typ=nx.DiGraph, dl=" ", clean=False):
         path = pre + "pge/samples/networks/web-"
         if network == "BS":
             path += "BerkStan.txt"
@@ -88,7 +88,14 @@ class Creator:
             path += "Google.txt"
         else:
             path = network
-        return SGraph(nx.read_edgelist(path, create_using=nx.DiGraph()))
+        res = nx.read_edgelist(path, create_using=typ(), delimiter=dl)
+
+        sub = max(nx.connected_components(res), key=len)
+        res = SGraph(res).subgraph(sub)
+
+        if clean:
+            res.del_attrs(["pos"])
+        return res
 
     @staticmethod
     def fire_forest(p_f, p_r, n, sg=2, fl=True):
@@ -144,7 +151,9 @@ class Creator:
             gr.del_edge(int(node2), int(node))
 
             un = rand_int(1, ln, 1)
-            gr.add_path([int(node2)] + [int(count + i) for i in np.arange(un)] + [int(node)])
+            gr.add_path(
+                [int(node2)] + [int(count + i) for i in np.arange(un)] + [int(node)]
+            )
             count = n + un
 
         actual(gr, sg)
